@@ -9,168 +9,158 @@ require      Exporter;
 our @ISA = qw (Exporter);
 our @EXPORT = qw (baseconfigure verbosity freezeverbosity);
 
-our $VERSION = "1.19";
+our $VERSION = "1.20";
 
 $Baseobject::directfeedback=0; # for downward compatibility
 $Baseobject::quickdeath=0;
 $Baseobject::verbosity=1;
 
 
-sub new
-{
-  my $pkg=shift;
-  my $param=shift;
+sub new {
+	my $pkg = shift;
+	my $param = shift;
+	my $self = bless {},$pkg;
 
-  my $self=bless {},$pkg;
+	$self->{LOCALSTATEDIR}= '';
+	$self->{SYSCONFDIR}   = '';
+	$self->{DATADIR}      = '';
+	$self->{PACKAGE}      = '';
 
-  $self->{LOCALSTATEDIR}='';
-  $self->{SYSCONFDIR}   ='';
-  $self->{DATADIR}      ='';
-  $self->{PACKAGE}      ='';
+	$self->{LOCALSTATEDIR} = $main::LOCALSTATEDIR if defined ($main::LOCALSTATEDIR);
+	$self->{SYSCONFDIR} = $main::SYSCONFDIR if defined ($main::SYSCONFDIR);
+	$self->{DATADIR} = $main::DATADIR if defined ($main::DATADIR);
+	$self->{PACKAGE} = $main::PACKAGE if defined ($main::PACKAGE);
 
-  $self->{LOCALSTATEDIR}=$main::LOCALSTATEDIR   if (defined ($main::LOCALSTATEDIR));
-  $self->{SYSCONFDIR}   =$main::SYSCONFDIR      if (defined ($main::SYSCONFDIR));
-  $self->{DATADIR}     = $main::DATADIR         if (defined ($main::DATADIR));
-  $self->{PACKAGE}     = $main::PACKAGE         if (defined ($main::PACKAGE));
+	unless ($self->{DATADIR}) {
+		$self->{DATADIR} = $main::datadir if defined ($main::datadir);
+	}
 
-  unless ( ($self->{DATADIR}))
-  {
-    $self->{DATADIR}=$main::datadir   if (defined ($main::datadir));
-  }
+	unless ($self->{SYSCONFDIR})  {
+		$self->{SYSCONFDIR} = $main::sysconfdir if defined($main::sysconfdir);
+	}
 
-  unless ( ($self->{SYSCONFDIR}))
-  {
-    $self->{SYSCONFDIR}=$main::sysconfdir   if (defined ($main::sysconfdir));
-  }
+	unless($self->{LOCALSTATEDIR}) {
+		$self->{LOCALSTATEDIR} = $main::localstatedir if defined($main::localstatedir);
+	}
 
-  unless ( ($self->{LOCALSTATEDIR}))
-  {
-    $self->{LOCALSTATEDIR}=$main::datadir   if (defined ($main::localstatedir));
-  }
+	$self->clearerror();
+	$self->clearfriendlyerror();
 
-  $self->clearerror();
-  $self->clearfriendlyerror();
+	$self->addfeatures ( {'allows'=>1, 'needs'=>1} );
+	$self->allows ({'name'=>'unnamed'});
 
-  $self->addfeatures ( {'allows'=>1, 'needs'=>1} );
-  $self->allows ({'name'=>'unnamed'});
-
-  #compatibility
-  $self->{public}=$self->{fields};
-  return $self;
+	#compatibility
+	$self->{public}=$self->{fields};
+	return $self;
 }
 
-sub features
-{
-  my $self=shift;
-  my $feature=shift;
+sub features{
+	my $self=shift;
+	my $feature=shift;
 
-  my $ptr=shift;
+	my $ptr=shift;
 
-  if ( ref($ptr) eq 'HASH')
-  {
-    foreach my $var (keys %$ptr)
-    {
-      #set the default
-      $self->{$feature}{$var}=$ptr->{$var};
-      $self->field($var,$ptr->{$var});
-    }
-  }
-  elsif( ref($ptr) eq 'ARRAY')
-  {
-    foreach my $var (@$ptr)
-    {
-      $self->{$feature}{$var}='*';
-    }
-  }
-  else # scalar
-  {
-    $self->{$feature}{$ptr}='*';
-  }
+	if ( ref($ptr) eq 'HASH') 
+	{
+		foreach my $var (keys %$ptr)
+		{
+			#set the default
+			$self->{$feature}{$var}=$ptr->{$var};
+			$self->field($var,$ptr->{$var});
+		}
+	}
+	elsif( ref($ptr) eq 'ARRAY')
+	{
+		foreach my $var (@$ptr)
+		{
+			$self->{$feature}{$var}='*';
+		}
+	}
+	else # scalar
+	{
+		$self->{$feature}{$ptr}='*';
+	}
 }
 
 sub allows
 {
-  my $self=shift;
-  $self->features( "allows",@_);
+	my $self=shift;
+	$self->features( "allows",@_);
 }
 
 sub needs
 {
-  my $self=shift;
-  $self->features( "needs",@_);
+	my $self=shift;
+	$self->features( "needs",@_);
 }
 
 sub addfeatures
 {
 
-  my $self=shift;
-  my $pFeatures=shift;
+	my $self=shift;
+	my $pFeatures=shift;
 
-  foreach my $feature (keys %$pFeatures)
-  {
-    $self->{features}->{$feature}=$pFeatures->{$feature};
-  }
+	foreach my $feature (keys %$pFeatures)
+	{
+		$self->{features}->{$feature}=$pFeatures->{$feature};
+	}
 }
 
 ##########################################
 sub stderror
 {
-  my $self=shift;
-  $self->error(@_);
-  print STDERR @_,"\n";
+	my $self=shift;
+	$self->error(@_);
+	print STDERR @_,"\n";
 }
 
-sub error
-{
-  my $self=shift;
+sub error {
+	my $self = shift;
 
-  if (@_ && $_[0])
-  {
-    confess (@_)if ($self->quickdeath);
-    $self->{error}.= join ('',@_);
-    return 0;
-  }
-  else
-  {
-    my ($package,$file,$line)=caller;
-    my $ret = "$package:$file:$line ". $self->{error};
-	return $ret;
-  }
+	if (@_ && $_[0]) {
+		confess (@_) if $self->quickdeath;
+		$self->{error} .= join ('',@_);
+		return 0;
+	} else {
+		my ($package, $file, $line) = caller;
+		return "$package:$file:$line " . $self->{error} if $self->{error};
+		return $self->{error};
+	}
 }
 
 sub clearerror
 {
-  my $self=shift;
-  $self->{error}="";
+	my $self=shift;
+	$self->{error}="";
 }
 
 sub friendlyerror
 {
-  my $self=shift;
+	my $self=shift;
 
-  if (@_ && $_[0])
-  {
-    $self->{friendlyerror}.= join ('',@_);
-    return 0;
-  }
-  else
-  {
-    if ($self->{friendlyerror})
-    {
-      return $self->{friendlyerror};
-    }
+	if (@_ && $_[0])
+	{
+		$self->{friendlyerror}.= join ('',@_);
+		return 0;
+	}
+	else
+	{
+		if ($self->{friendlyerror})
+		{
+			return $self->{friendlyerror};
+		}
 
-    else
-    {
-      return $self->{error};
-    }
-  }
+		else
+		{
+			return $self->{error};
+		}
+	}
 }
 
 sub clearfriendlyerror
 {
-  my $self=shift;
-  $self->{friendlyerror}="";
+	my $self=shift;
+	$self->{friendlyerror}="";
 }
 
 
@@ -179,126 +169,126 @@ sub clearfriendlyerror
 
 sub field
 {
-  my $self=shift;
-  my $var=shift;
-  my $value=shift;
+	my $self=shift;
+	my $var=shift;
+	my $value=shift;
 
-  $var='value' unless ($var);
-  return $self->fields($var) if ref ($var) eq 'HASH';
+	$var='value' unless ($var);
+	return $self->fields($var) if ref ($var) eq 'HASH';
 
-  unless (exists ($self->{allowallfields}))
-  {
-    unless  ( (exists  $self->{allows}->{$var}) ||
-	    (exists  $self->{needs}->{$var})
-	    )
-    {
-      $self->stderror("field: unknown variable: \"$var\", accepted vars are:\n\t",
-		 $self->showallows());
-      return "";
-    }
-  }
+	unless (exists ($self->{allowallfields}))
+	{
+		unless  ( (exists  $self->{allows}->{$var}) ||
+				  (exists  $self->{needs}->{$var})
+			)
+		{
+			$self->stderror("field: unknown variable: \"$var\", accepted vars are:\n\t",
+							$self->showallows());
+			return "";
+		}
+	}
 
 
-  return defined($value) ? $self->{fields}{$var} = $value
-      : $self->{fields}{$var};
+	return defined($value) ? $self->{fields}{$var} = $value
+		: $self->{fields}{$var};
 }
 
 
 sub fields
 {
-  my $self=shift;
-  my $pp=shift;
+	my $self=shift;
+	my $pp=shift;
 
-  unless ($pp)
-  {
-    return $self->{fields};
-  }
+	unless ($pp)
+	{
+		return $self->{fields};
+	}
 
-  my $type=ref($pp);
+	my $type=ref($pp);
 
-  if ( $type eq 'HASH')
-  {
+	if ( $type eq 'HASH')
+	{
 
-    #
-    # strict order of scalars interpretation
-    #
-    my %cp=%$pp;
-    my @keylist;
+		#
+		# strict order of scalars interpretation
+		#
+		my %cp=%$pp;
+		my @keylist;
 
 #    warn $self->inspect($pp);
 
-    foreach my $thing (qw (name options default ))
-    {
-      if (exists ($pp->{$thing}))
-      {
-	push (@keylist,$thing) ;
-	delete $cp{$thing};
-      }
-    }
-    push (@keylist,  (keys %cp));
+		foreach my $thing (qw (name options default ))
+		{
+			if (exists ($pp->{$thing}))
+			{
+				push (@keylist,$thing) ;
+				delete $cp{$thing};
+			}
+		}
+		push (@keylist,  (keys %cp));
 
-    foreach my $params (@keylist)
-    {
-      $self->field($params,$pp->{$params});
-    }
-    return ;#$self->checkneeds();
-  }
-  elsif($type eq 'ARRAY')
-  {
-    my @retlist=();
-    foreach my $el (@$pp)
-    {
-      push (@retlist , $self->field($el));
-    }
-    return @retlist;
-  }
-  elsif (!$type && $pp)
-  {
-    #scalar
-    return $self->field($pp);
-  }
+		foreach my $params (@keylist)
+		{
+			$self->field($params,$pp->{$params});
+		}
+		return ;#$self->checkneeds();
+	}
+	elsif($type eq 'ARRAY')
+	{
+		my @retlist=();
+		foreach my $el (@$pp)
+		{
+			push (@retlist , $self->field($el));
+		}
+		return @retlist;
+	}
+	elsif (!$type && $pp)
+	{
+		#scalar
+		return $self->field($pp);
+	}
 
 }
 
 sub checkneeds
 {
 
-  my $self=shift;
-  if (exists ($self->{needs}))
-  {
-    foreach my $needs (keys %{$self->{needs}})
-    {
-      if (!defined ($self->field($needs)))
-      {
-	return $self->error("missing requirement: \"$needs\"\n");
-      }
-    }
-  }
-  return $self;
+	my $self=shift;
+	if (exists ($self->{needs}))
+	{
+		foreach my $needs (keys %{$self->{needs}})
+		{
+			if (!defined ($self->field($needs)))
+			{
+				return $self->error("missing requirement: \"$needs\"\n");
+			}
+		}
+	}
+	return $self;
 }
 
 
 sub inspect {
-  my $self=shift;
-  my $string = Dumper(@_);
-  my ($package, $file, $line) = caller;
-  $string.= " Inspect called at: $file line $line\n";
+	my $self=shift;
+	my $string = Dumper(@_);
+	my ($package, $file, $line) = caller;
+	$string.= " Inspect called at: $file line $line\n";
 }
 
 
 
 sub showallows
 {
-  my $self=shift;
-  return  join (' ',(  (keys %{$self->{needs}}), keys %{$self->{allows}})),"\n";
+	my $self=shift;
+	return  join (' ',(  (keys %{$self->{needs}}), keys %{$self->{allows}})),"\n";
 }
 
 
 sub dumpfields
 {
-  my $self=shift;
+	my $self=shift;
 
-  return $self->inspect($self->{fields});
+	return $self->inspect($self->{fields});
 }
 
 
@@ -307,34 +297,34 @@ sub dumpall
 #
 #call in despair only
 #
-  my $self=shift;
-  use Dumpvalue;
-  my $dumper = new Dumpvalue;
-  $dumper->set(globPrint => 1);
-  $dumper->dumpValue(\*::);
-  $dumper->dumpvars('main');
+	my $self=shift;
+	use Dumpvalue;
+	my $dumper = new Dumpvalue;
+	$dumper->set(globPrint => 1);
+	$dumper->dumpValue(\*::);
+	$dumper->dumpvars('main');
 }
 
 sub iscgi
 {
-  return $ENV{'SERVER_ADDR'};
+	return $ENV{'SERVER_ADDR'};
 }
 
 
 sub basefilename
 {
-  # returns basefilename
-  my $self=shift;
-  my $basename=$0;
+	# returns basefilename
+	my $self=shift;
+	my $basename=$0;
 
-  if ($basename =~ /\//)
-  {
-    #chop off path, note: it might contain nasty dots, get rid of them now
-    $basename=~s/(^.*\/)//; #$path=$1;
-  }
+	if ($basename =~ /\//)
+	{
+		#chop off path, note: it might contain nasty dots, get rid of them now
+		$basename=~s/(^.*\/)//; #$path=$1;
+	}
 #  $basename=~s/\..*?$//;
-  chomp $basename;
-  return $basename;
+	chomp $basename;
+	return $basename;
 }
 
 
@@ -342,73 +332,73 @@ sub basefilename
 
 sub baseconfigure
 {
-  my $ptr=shift;
-  if (ref ($ptr) ne 'HASH' )
-  {
-    warn "Baseobject%ERROR baseconfigure needs ref to HASH, not: $ptr";
-    return;
-  }
-  for  (keys (%$ptr))
-  {
-    if (/^quickdeath$/)
-    {
-      $Baseobject::quickdeath=$ptr->{$_};
-      next;
-    }
-    elsif (/^verbosity$/)
-    {
-      $Baseobject::verbosity=$ptr->{$_};
-      next;
-    }
-    elsif (/^directfeedback$/)
-    {
-      $Baseobject::directfeedback=$ptr->{$_};
-      next;
-    }
+	my $ptr=shift;
+	if (ref ($ptr) ne 'HASH' )
+	{
+		warn "Baseobject%ERROR baseconfigure needs ref to HASH, not: $ptr";
+		return;
+	}
+	for  (keys (%$ptr))
+	{
+		if (/^quickdeath$/)
+		{
+			$Baseobject::quickdeath=$ptr->{$_};
+			next;
+		}
+		elsif (/^verbosity$/)
+		{
+			$Baseobject::verbosity=$ptr->{$_};
+			next;
+		}
+		elsif (/^directfeedback$/)
+		{
+			$Baseobject::directfeedback=$ptr->{$_};
+			next;
+		}
 
-    warn "Baseobject%ERROR baseconfigure accepts only: directfeedback, verbosity and quickdeath\n";
-    warn "NOT: $_\n";
+		warn "Baseobject%ERROR baseconfigure accepts only: directfeedback, verbosity and quickdeath\n";
+		warn "NOT: $_\n";
 
-    next;
-  }
+		next;
+	}
 }
 
 sub quickdeath
 {
-  my $self=shift;
+	my $self=shift;
 
-  $Baseobject::quickdeath =1 if (!defined ($Baseobject::quickdeath)&& (!@_));
+	$Baseobject::quickdeath =1 if (!defined ($Baseobject::quickdeath)&& (!@_));
 
- @_ ? $Baseobject::quickdeath = shift
-      : $Baseobject::quickdeath;
+	@_ ? $Baseobject::quickdeath = shift
+		: $Baseobject::quickdeath;
 }
 
 
 sub freezeverbosity
 {
-  $Baseobject::freezeverbosity =0 if (!defined ($Baseobject::freezeverbosity)&& (!@_));
-  @_ ? $Baseobject::freezeverbosity=shift: $Baseobject::freezeverbosity;
+	$Baseobject::freezeverbosity =0 if (!defined ($Baseobject::freezeverbosity)&& (!@_));
+	@_ ? $Baseobject::freezeverbosity=shift: $Baseobject::freezeverbosity;
 }
 
 sub verbosity
 {
 
-  if (@_)
-  {
-    my $ref=ref ( $_[0]);
-    if ( $ref)
-    {
-      warn "WRONG PARAMETER FOR Baseobject::verbosity: should be 0, 1, 2, 3 or 4, not: type \"$_[0]\"\n";
-      warn "Did you by any change call \$object->verbosity(int) ? in stead of Baseobject::verobosity(int)\n";
-      warn "verbosity setting IGNORED\n";
-      return;
-    }
-  }
+	if (@_)
+	{
+		my $ref=ref ( $_[0]);
+		if ( $ref)
+		{
+			warn "WRONG PARAMETER FOR Baseobject::verbosity: should be 0, 1, 2, 3 or 4, not: type \"$_[0]\"\n";
+			warn "Did you by any change call \$object->verbosity(int) ? in stead of Baseobject::verobosity(int)\n";
+			warn "verbosity setting IGNORED\n";
+			return;
+		}
+	}
 
-  $Baseobject::verbose =1 if (!defined ($Baseobject::verbose)&& (!@_));
+	$Baseobject::verbose =1 if (!defined ($Baseobject::verbose)&& (!@_));
 
- (@_ && (!freezeverbosity) ) ? $Baseobject::verbosity = shift
-      : $Baseobject::verbosity;
+	(@_ && (!freezeverbosity) ) ? $Baseobject::verbosity = shift
+		: $Baseobject::verbosity;
 
 }
 
@@ -416,113 +406,96 @@ sub verbosity
 
 sub verbose
 {
-  my $self=shift;
-  my ($package,$file,$line)=caller;
+	my $self=shift;
+	my ($package,$file,$line)=caller;
 
-  if (verbosity())
-  {
-    foreach my $el (@_)
-    {
-      my $location='';
-      $location=   " at $file line $line" unless ($el=~/\n$/);
-      print STDERR " $el$location";
-    }
-  }
+	if (verbosity())
+	{
+		foreach my $el (@_)
+		{
+			my $location='';
+			$location=   " at $file line $line" unless ($el=~/\n$/);
+			print STDERR " $el$location";
+		}
+	}
 }
 
 sub debug
 {
-  my $self=shift;
-  if (verbosity()>=2)
-  {
-    my ($package,$file,$line)=caller;
+	my $self=shift;
+	if (verbosity()>=2)
+	{
+		my ($package,$file,$line)=caller;
 
-    foreach my $el (@_)
-    {
-      my $location='';
-      $location=   " at $file line $line" unless ($el=~/\n$/);
-      print STDERR "  $el$location";
-    }
-  }
+		foreach my $el (@_)
+		{
+			my $location='';
+			$location=   " at $file line $line" unless ($el=~/\n$/);
+			print STDERR "  $el$location";
+		}
+	}
 }
 
 
 sub expandvars
 {
-  my $self=shift;
-  my $ptrstring=shift;
-  my $h=shift;
+	my $self=shift;
+	my $ptrstring=shift;
+	my $h=shift;
 
-  unless (ref ($ptrstring) eq 'SCALAR' )
-  {
-    carp "EXPANDVARS: first parameter to should be a ref to a scalar!";
-    return undef;
-  }
+	unless (ref ($ptrstring) eq 'SCALAR' ) {
+		carp "EXPANDVARS: first parameter to should be a ref to a scalar!";
+		return undef;
+	}
 
-  unless (ref ($h) eq 'HASH' )
-  {
-    carp "EXPANDVARS: second parameter should be a ref to a hash";
-    return undef;
-  }
+	unless (ref ($h) eq 'HASH' ) {
+		carp "EXPANDVARS: second parameter should be a ref to a hash";
+		return undef;
+	}
 
-  unless (defined ($$ptrstring))
-  {
-    carp "EXPANDVARS: ptr to string undefined";
-    return $$ptrstring;
-  }
+	unless (defined ($$ptrstring)) {
+		carp "EXPANDVARS: ptr to string undefined";
+		return $$ptrstring;
+	}
 
- fail:
-  #recursive expand $vars from the fields
-#  my $varexpr='\$([\w\{\}]+)';
+  fail:
+	# recursive expand $vars from the fields
+	# my $varexpr='\$([\w\{\}]+)';
 
-  while ( $$ptrstring =~ /\$([\w\{\}]+)/s)
-  {
-    my @varlist=$$ptrstring =~ /\$([\w\{\}]+)/sg;
-#
-#    warn "varlist: @varlist";
-    foreach my $var ( @varlist)
-    {
-#      warn "initial var: $var";
-      # save curly brackets and slash them
-      my $cpvar=$var; $cpvar =~ s/(\W)/\\$1/g;
+	while ($$ptrstring =~ /\$([\w\{\}]+)/s) {
+		my @varlist = $$ptrstring =~ /\$([\w\{\}]+)/sg;
+		# warn "varlist: @varlist";
+		foreach my $var (@varlist) {
+			# warn "initial var: $var";
+			# save curly brackets and slash them
+			my $cpvar = $var; $cpvar =~ s/(\W)/\\$1/g;
+			# remove curly brackets if they are there
+			my ($secvar) = $var =~ /\{([^\}]+)\}/;
+			$secvar = $var unless ($secvar);
 
-      # remove curly brackets if they are there
-      my ($secvar)=  $var =~ /\{([^\}]+)\}/;
-
-      $secvar=$var unless ($secvar);
-
-      if (exists ($h->{$secvar}))
-      {
-        #emacs indentation realy screws up here
-         my @x;
-	 if ($var =~ /\}/)
-	 {
-	    @x=$$ptrstring =~ s/\$\{$secvar\}/$h->{$secvar}/sg;
-	 }
-	  else
-         {
-	    @x=$$ptrstring =~ s/\$$cpvar([\W\b]|$)/$h->{$secvar}$1/sg;
-	 }
-         unless (@x)
-         {
-    	    cluck "Substitute failed: for \"\$$var\" \nQUITING expandvars";
-  	    carp "$$ptrstring";
-  	    return 0;
-         }
-	 else
-	 {
-	   #show this only at level 2 or higher
-	   $self->debug("Expandvars: result: $$ptrstring") if (verbosity()>2)
-	 }
-      }
-      else
-      {
-        cluck "Could not substitute: \"\$$var\" \nQUITING expandvars";
-        return 0;
-      }
-    }
-  }
-  return $$ptrstring;
+			if (exists ($h->{$secvar})) {
+				#emacs indentation realy screws up here
+				my @x;
+				if ($var =~ /\}/) {
+					@x=$$ptrstring =~ s/\$\{$secvar\}/$h->{$secvar}/sg;
+				} else {
+					@x=$$ptrstring =~ s/\$$cpvar([\W\b]|$)/$h->{$secvar}$1/sg;
+				}
+				unless (@x) {
+					cluck "Substitute failed: for \"\$$var\" \nQUITING expandvars";
+					carp "$$ptrstring";
+					return 0;
+				} else {
+					#show this only at level 2 or higher
+					$self->debug("Expandvars: result: $$ptrstring") if (verbosity()>2)
+				}
+			} else {
+				cluck "Could not substitute: \"\$$var\" \nQUITING expandvars";
+				return 0;
+			}
+		}
+	}
+	return $$ptrstring;
 }
 
 
